@@ -45,7 +45,7 @@ void uwsgi_log(const char *fmt, ...) {
 		}
 		else {
 			gettimeofday(&tv, NULL);
-#ifdef __sun__
+#if defined(__sun__) && !defined(__clang__)
 			ctime_r((const time_t *) &tv.tv_sec, ctime_storage, 26);
 #else
 			ctime_r((const time_t *) &tv.tv_sec, ctime_storage);
@@ -97,7 +97,7 @@ void uwsgi_log_verbose(const char *fmt, ...) {
 	}
 	else {
 		gettimeofday(&tv, NULL);
-#ifdef __sun__
+#if defined(__sun__) && !defined(__clang__)
 		ctime_r((const time_t *) &tv.tv_sec, ctime_storage, 26);
 #else
 		ctime_r((const time_t *) &tv.tv_sec, ctime_storage);
@@ -616,7 +616,7 @@ void uwsgi_logit_simple(struct wsgi_request *wsgi_req) {
 			break;	
 	}
 
-#ifdef __sun__
+#if defined(__sun__) && !defined(__clang__)
 	ctime_r((const time_t *) &wsgi_req->start_of_request_in_sec, time_request, 26);
 #else
 	ctime_r((const time_t *) &wsgi_req->start_of_request_in_sec, time_request);
@@ -1035,7 +1035,7 @@ static ssize_t uwsgi_lf_epoch(struct wsgi_request * wsgi_req, char **buf) {
 
 static ssize_t uwsgi_lf_ctime(struct wsgi_request * wsgi_req, char **buf) {
 	*buf = uwsgi_malloc(26);
-#ifdef __sun__
+#if defined(__sun__) && !defined(__clang__)
 	ctime_r((const time_t *) &wsgi_req->start_of_request_in_sec, *buf, 26);
 #else
 	ctime_r((const time_t *) &wsgi_req->start_of_request_in_sec, *buf);
@@ -1060,6 +1060,19 @@ static ssize_t uwsgi_lf_ltime(struct wsgi_request * wsgi_req, char **buf) {
 	return ret;
 }
 
+static ssize_t uwsgi_lf_ftime(struct wsgi_request * wsgi_req, char **buf) {
+	if (!uwsgi.log_strftime) {
+		return uwsgi_lf_ltime(wsgi_req, buf);
+	}
+	*buf = uwsgi_malloc(64);
+	time_t now = wsgi_req->start_of_request / 1000000;
+	size_t ret = strftime(*buf, 64, uwsgi.log_strftime, localtime(&now));
+	if (ret == 0) {
+		*buf[0] = 0;
+		return 0;
+	}
+	return ret;
+}
 
 static ssize_t uwsgi_lf_micros(struct wsgi_request * wsgi_req, char **buf) {
 	*buf = uwsgi_num2str(wsgi_req->end_of_request - wsgi_req->start_of_request);
@@ -1246,6 +1259,11 @@ void uwsgi_add_logchunk(int variable, int pos, char *ptr, size_t len) {
 			logchunk->func = uwsgi_lf_ltime;
 			logchunk->free = 1;
 		}
+		else if (!uwsgi_strncmp(ptr, len, "ftime", 5)) {
+			logchunk->type = 3;
+			logchunk->func = uwsgi_lf_ftime;
+			logchunk->free = 1;
+		}
 		else if (!uwsgi_strncmp(ptr, len, "ctime", 5)) {
 			logchunk->type = 3;
 			logchunk->func = uwsgi_lf_ctime;
