@@ -1805,6 +1805,7 @@ PyObject *py_uwsgi_send_spool(PyObject * self, PyObject * args, PyObject *kw) {
 		if (PyString_Check(pybody)) {
 			body = PyString_AsString(pybody);
 			body_len = PyString_Size(pybody);
+			Py_INCREF(pybody);
 			uwsgi_py_dict_del(spool_dict, "body");
 		}
 	}
@@ -1896,6 +1897,10 @@ PyObject *py_uwsgi_send_spool(PyObject * self, PyObject * args, PyObject *kw) {
 	i = spool_request(uspool, spool_filename, uwsgi.workers[0].requests + 1, async_id, spool_buffer, cur_buf - spool_buffer, priority, at, body, body_len);
 
 	UWSGI_GET_GIL
+
+	if (pybody) {
+		Py_DECREF(pybody);
+	}
 	
 	if (priority) {
 		free(priority);
@@ -3104,6 +3109,35 @@ PyObject *py_uwsgi_cache_num(PyObject * self, PyObject * args) {
 
 }
 
+PyObject *py_uwsgi_cache_keys(PyObject * self, PyObject * args) {
+	char *cache = NULL;
+        struct uwsgi_cache_item *uci = NULL;
+        uint64_t pos = 0;
+
+        if (!PyArg_ParseTuple(args, "|s:cache_keys", &cache)) {
+                return NULL;
+        }
+
+	struct uwsgi_cache *uc = uwsgi_cache_by_name(cache);
+	if (!uc) {
+		return PyErr_Format(PyExc_ValueError, "no local uWSGI cache available");
+	}
+
+	PyObject *l = PyList_New(0);
+
+	uwsgi_rlock(uc->lock);
+        for(;;) {
+                uci = uwsgi_cache_keys(uc, &pos, &uci);
+                if (!uci) break;
+		PyObject *ci = PyString_FromStringAndSize(uci->key, uci->keysize);
+		PyList_Append(l, ci);
+		Py_DECREF(ci);
+        }
+	uwsgi_rwunlock(uc->lock);
+	return l;
+}
+
+
 static PyMethodDef uwsgi_cache_methods[] = {
 	{"cache_get", py_uwsgi_cache_get, METH_VARARGS, ""},
 	{"cache_set", py_uwsgi_cache_set, METH_VARARGS, ""},
@@ -3116,6 +3150,7 @@ static PyMethodDef uwsgi_cache_methods[] = {
 	{"cache_mul", py_uwsgi_cache_mul, METH_VARARGS, ""},
 	{"cache_div", py_uwsgi_cache_div, METH_VARARGS, ""},
 	{"cache_num", py_uwsgi_cache_num, METH_VARARGS, ""},
+	{"cache_keys", py_uwsgi_cache_keys, METH_VARARGS, ""},
 	{NULL, NULL},
 };
 
